/* 
 * Nazev:	Genealogicky expertni system
 * Tda:	Rodokmen
 * Autor:	Antnonin Neumann
 * Datum:	13. 4. 2014
 * Verze:	1
 */
package genealogy;

import java.util.LinkedList;

/**
 * Trida reprezentujici rodokmen (rodiny strom) a pravidla pro expertni system
 * 
 * @author Antonin Neumann
 *
 */
public class Lineage {

	public Person root;
	public LinkedList<Relation> rules;
	
	public Lineage(){
		root = new Person(0, "First", "Acestor");
		rules = new LinkedList<Relation>();
	}
	
	/**
	 * Prida novou osobu do rodineho stromu
	 * 
	 * @param data String Struktura: ID;NAME;LASTNAME;GENDER;MOTHERID;FATHERID;PARTNERID
	 * @return Person - nove vytvorenou osobu nebo existujici nalezenou osobu
	 */
	public Person addPerson(String data){
		String[] attributes = data.split(";");
		if(attributes.length < 6){ new Exception("Bad input format"); }
		
		System.out.println("Person data: "+data);
		
		//oriznuti bilych znaku pro kazdy atribut
		for (String attr : attributes) { attr = attr.trim(); }
		
		//nejprve zkusim, jestli uz dana osoba existuje
		Person current;
		if((current = findPersonById(Integer.parseInt(attributes[0]))) != null){
			System.out.println("Found: "+current.toString());
			//pokud je osoba jen torzo, tak k ni doplnim udaje
			if(current.getName().compareToIgnoreCase("torso") == 0){
				current.setName(attributes[1]);
				current.setLastname(attributes[2]);
				current.setGender(Gender.getFromString(attributes[3]));
				System.out.println("Update: "+current.toString());
			}
		}else {
			//vytvorim torzo nove osoby
			current = new Person(Integer.parseInt(attributes[0]), attributes[1], attributes[2], Gender.getFromString(attributes[3]));
			System.out.println("Create: "+current.toString());
			
			//pri novem vytvoreni osoby nastavime jako predka root
			current.setMom(root);
			root.addChild(current);
		}
		
		Person subPerson;
		int idSubPerson;
		//pridani matky
		try{
			idSubPerson = Integer.parseInt(attributes[4]);
			if((subPerson = findPersonById(idSubPerson)) == null){ //osoba (matka) neexistuje => vytvorime ji
				subPerson = new Person(idSubPerson, "torso", "torso", Gender.FEMALE);
			}
			current.setMom(subPerson); //nastavime current osobe nove vytvorenou matku
			subPerson.addChild(current); //zaroven current osobu jako dite matky
			System.out.println("Update mom: "+current.toString());
			
			//pokud matka nema matku a neni uz dite root matky tak ji pridame
			if(!isRootChild(subPerson) && subPerson.getMom() == null){
				subPerson.setMom(root);
				root.addChild(subPerson);
			}
			
			System.out.println("Mother of "+current.getId()+ " is " +current.getMom().toString());
			
			//pokud byla current osoba vedena jako dite root matky, tak ji nyni odebereme
			if(isRootChild(current)){
				root.removeChild(current);
				System.out.println("Remove "+current.toPrint()+" from root children");
			}
			
		}catch(NumberFormatException e){
			System.err.println("Osoba "+current.getId()+" nema matku.");
		}
		
		//pridani otce
		try{
			idSubPerson = Integer.parseInt(attributes[5]);
			if((subPerson = findPersonById(idSubPerson)) == null){ //osoba (otec) neexistuje => vytvorime jej
				subPerson = new Person(idSubPerson, "torso", "torso", Gender.MALE);
			}
			current.setDad(subPerson); //nastavime current osobe nove vytvoreneho otce
			subPerson.addChild(current); //zaroven current osobu jako dite otce
			System.out.println("Update dad: "+current.toString());
			
			//pokud otec nema matku a neni uz dite root matky tak ho pridame
			if(!isRootChild(subPerson) && subPerson.getMom() == null){
				subPerson.setMom(root);
				root.addChild(subPerson);
			};
			
			System.out.println("Father of "+current.getId()+ " is " +current.getDad().toString());
			
		}catch(NumberFormatException e){
			System.err.println("Osoba "+current.getId()+" nema otce.");
		}
		
		//pridani partnera
		try{
			idSubPerson = Integer.parseInt(attributes[6]);
			if((subPerson = findPersonById(idSubPerson)) == null){ //osoba (partner) neexistuje => vytvorime jej
				subPerson = new Person(idSubPerson, "torso", "torso", Gender.getOpposite(current.getGender()));
			}
			current.setPartner(subPerson);
			subPerson.setPartner(current);

			
		}catch(NumberFormatException e){
			System.err.println("Osoba "+current.getId()+" nema partnera.");
		}
		
		System.out.println();
		return current;
	}

	/**
	 * Prida nove pravidlo
	 * 
	 * @param data String Struktura: ID;NAME;MALE_MALE;MALE_4TH_NAME;FEMALE_MALE;FEMALE_4TH_NAME;RULE
	 * @return <strong>TRUE</strong> pokud bylo pravidlo pidno, jinak <strong>FALSE</strong>
	 */
	public boolean addRelation(String data){
		String[] attr = data.split(";");
		if(attr.length < 7){ new Exception("Bad input format"); }
		
		//oriznuti bilych znaku pro kazdy atribut
		for (String a : attr) { a = a.trim(); }
		
		System.out.println("Rule data: "+data);
		Relation newRule = new Relation(Integer.parseInt(attr[0]), attr[2], attr[3], attr[4], attr[5], attr[1], attr[6]);
		if(!rules.contains(newRule)){
			rules.add(newRule);
			return true;
		}
		return false;
	}
	
	/**
	 * Prida nove pravidlo
	 * 
	 * @param r Relation {@link genealogy.Relation}
	 * @return <strong>TRUE</strong> pokud bylo pravidlo pidno, jinak <strong>FALSE</strong>
	 */
	public boolean addRelation(Relation r){
		if(!rules.contains(r)){
			rules.add(r);
			return true;
		}
		return false;
	}
	
	/**
	 * Vypise strom od zadane osoby. 
	 * 
	 * @param root Person
	 */
	public void printLineage(Person root) { printLineage(root, "|", true, false); }
	
	/**
	 * Vypise strom pokrevnym pribuznych od zadane osoby
	 * 
	 * @param root Person
	 */
	public void printBloodLine(Person root){ printLineage(root, "|", true, true); }
	
	/**
	 * Vypise strom od zadane osoby
	 * 
	 * @param root Person
	 * @param indent String Odrazna, vypisuje se pred osobou pro hezci vystup
	 * @param first boolean TRUE pokud se jedna o koren stromu
	 * @param bloodLine boolena TRUE pro vypis pouze pokrevnich pribuznych
	 */
	private void printLineage(Person root, String indent, boolean first, boolean bloodLine){
		if(first){
			System.out.println(root.toPrint());
		}else {
			System.out.println(indent+ " " +root.toPrint());
		}
		
		if(!bloodLine && root.getPartner() != null){
			System.out.println(String.format("%1$"+indent.length()+"s", " ") + " \\ " +root.getPartner().toPrint());
		}
		LinkedList<Person> children = root.getChildren();
		if(!children.isEmpty()){
			for (Person child : children) {
				printLineage(child, indent+"--", false, bloodLine);
			}
		}
	}
	
	/**
	 * Vypise seznam vsech pravidel
	 */
	public void printRules(){
		if(rules.isEmpty()){
			System.out.println("Pravidla jsou prazdna!");
		}else {
			for (Relation rule : rules) {
				System.out.println(rule);
			}
		}
	}
	
	/**
	 * Najde osobu podle ID
	 * 
	 * @param id integer - id hledane osoby
	 * @return Person
	 */
	public Person findPersonById(int id){ return findPersonById(id, this.root); }
	private Person findPersonById(int id, Person root){
		if (root.getId() == id){
	        return root;
		}else {
			Person partner = root.getPartner();
			if(partner != null && partner.getId() == id){
				return root.getPartner();
			}
		}
		
		LinkedList<Person> children = root.getChildren(); 
	    if(!children.isEmpty()){
	    	Person finded;
		    for (Person child : children) {         
		    	if((finded = findPersonById(id, child)) != null)
		    		return finded;
		    }
	    }
	    return null;
	}
	
	/**
	 * Metoda zjisti, jesli je zadana osoba potomkem "prvniho predka"
	 * @param p Person
	 * @return <strong>TRUE</strong> pokud je osoba potomkem "prvniho predka", jinak <strong>FALSE</strong>
	 */
	private boolean isRootChild(Person p){
		LinkedList<Person> children = root.getChildren();
		for (Person child : children) {
			if(child.equals(p))
				return true;
		}
		return false;
	}

	public Relation findRelation(int relationId){
		for (Relation rel : this.rules) {
			if(rel.getId() == relationId){
				return rel;
			}
		}
		return null;
	}
	
	public String getRelation(Person a, Person b){
		return null;
	}
	
	public LinkedList<Person> getPerson(Person p, Relation r){
		char[] rules = r.getRule().toCharArray();
		LinkedList<Person> stack = new LinkedList<Person>();
		LinkedList<Person> nextStack = new LinkedList<Person>();
		LinkedList<Person> not = new LinkedList<Person>();
		LinkedList<Person> tmp = new LinkedList<Person>();

		stack.add(p);
		for(int i = 0; i < rules.length; i++){
			if(stack.isEmpty()) return null;
			
			if(i%3 == 0){
				not.removeAll(not);
				not.addAll(stack);
			}
			
			switch (rules[i]) {
			case 'P':
				for (Person person : stack) {
					tmp = person.getParent();
					for (Person temp : tmp) {
						if(!nextStack.contains(temp))
							nextStack.add(temp);
					}
					//nextStack.addAll(person.getParent());
				}
				break;
			case 'C':
				for (Person person : stack) {
					tmp = person.getChildren();
					for (Person temp : tmp) {
						if(!nextStack.contains(temp))
							nextStack.add(temp);
					}
					//nextStack.addAll(person.getChildren());
				}
				break;
			case 'N':
				nextStack.addAll(stack);
				nextStack.removeAll(not);
				break;
			}
			stack.removeAll(stack);
			stack.addAll(nextStack);
			nextStack.removeAll(nextStack);
		}
		return stack;
	}
	
	public int countPersonOfRelation(Person p, Relation r){
		if(getPerson(p,r) == null){
			return 0;
		}
		return  getPerson(p, r).size();
	}
}
