/* 
 * Nazev:	Genealogicky expertni system
 * Tda:	Osoba
 * Autor:	Antnonin Neumann
 * Datum:	13. 4. 2014
 * Verze:	2
 */
package genealogy;

import java.util.LinkedList;

/**
 * Objekt reprezentujici osobu z rodokmenu
 * 
 * @author Antonin Neumann
 *
 */
public class Person {

	public static final int MAX_AGE = 150;
	
	private int id;
	private String name;
	private String lastname;
	private int age;
	private Gender gender;
	private boolean alive;
	private Person dad = null;
	private Person mom = null;
	private Person partner = null;
	private LinkedList<Person> children = new LinkedList<Person>();
	
	/**
	 * Minimal person constructor
	 * 
	 * @param id Integer - jedinecny identifikator
	 * @param name String - krestni jmeno
	 * @param lastname String - prijmeni
	 */
	public Person(int id, String name, String lastname){
		this(id, name, lastname, Gender.MALE);
	}
	
	/**
	 * 
	 * @param id Integer - jedinecny identifikator
	 * @param name String - krestni jmeno
	 * @param lastname String - prijmeni
	 * @param sex Gender - pohlavi
	 */
	public Person(int id, String name, String lastname, Gender sex){
		this(id, name, lastname, -1, sex);
	}
	
	/**
	 * 
	 * @param id Integer - jedinecny identifikator
	 * @param name String - krestni jmeno
	 * @param lastname String - prijmeni
	 * @param age Integer - vek
	 * @param sex Gender - pohlavi
	 */
	public Person(int id, String name, String lastname, int age, Gender sex){
		this(id, name, lastname, age, sex, true);
	}
	
	/**
	 * 
	 * @param id Integer - jedinecny identifikator
	 * @param name String - krestni jmeno
	 * @param lastname String - prijmeni
	 * @param age Integer - vek
	 * @param sex Gender - pohlavi
	 * @param alive Boolean - osoba zije
	 */
	public Person(int id, String name, String lastname, int age, Gender sex, boolean alive){
		this(id, name, lastname, age, sex, alive, null, null, null);
	}

	/**
	 * Full person constructor
	 * 
	 * @param id Integer - jedinecny identifikator
	 * @param name String - krestni jmeno
	 * @param lastname String - prijmeni
	 * @param age Integer - vek
	 * @param sex Gender - pohlavi
	 * @param alive Boolean - osoba zije
	 * @param dad Person - otec
	 * @param mom Person - matka
	 * @param partner Person - partner
	 */
	public Person(int id, String name, String lastname, int age, Gender sex, boolean alive, Person dad, Person mom, Person partner){
		this.id = id;
		this.name = name;
		this.lastname = lastname;
		this.age = age;
		this.gender = sex;
		this.alive = alive;
		this.dad = dad;
		this.mom = mom;
		this.partner = partner;
	}
	
	
	/**
	 * Vrati ID osoby
	 * @return integer ID
	 */
	public int getId() { return id; }
	
	/**
	 * <strong>TRUE</strong> je-li osoba stale nazivu, jinak <strong>FALSE</strong>
	 * @return boolean
	 */
	public boolean isAlive() { return alive; }
	
	/**
	 * <strong>TRUE</strong> je-li osoba jiz posmrti, jinak <strong>FALSE</strong>
	 * @return boolean
	 */
	public boolean isDead(){ return !this.alive; }

	/**
	 * Nastaveni, zda-li je osoba ziva nebo posmrti
	 * @param alive boolean
	 */
	public void setAlive(boolean alive) { this.alive = alive; }

	/**
	 * <strong>TRUE</strong> je-li  osoba muz, jinak <strong>FALSE</strong>
	 * @return boolean
	 */
	public boolean isMale() { return this.gender == Gender.MALE; }
	
	/**
	 * <strong>TRUE</strong> je-li  osoba zena, jinak <strong>FALSE</strong>
	 * @return boolean
	 */
	public boolean isFemale() { return this.gender == Gender.FEMALE; }
	
	/**
	 * Pohlavi osboy
	 * @return String
	 */
	public Gender getGender(){ return this.gender; }
	
	public void setGender(Gender sex){ this.gender = sex; }
	
	/**
	 * Jmeno osoby
	 * @return String
	 */
	public String getName() { return name; }
	
	public void setName(String name){ this.name = name; }
	
	/**
	 * Prijmeni osoby
	 * @return String
	 */
	public String getLastname() { return lastname; }

	/**
	 * Umoznuje nastavit prijmeni osoby (zejmena nove provdane zeny).
	 * @param lastname String
	 */
	public void setLastname(String lastname) { this.lastname = lastname; }
	
	/**
	 * Cele jmeno osoby
	 * @return String (getName() + getLastName())
	 */
	public String getFullName(){ return name + " " + lastname; }

	/**
	 * Vek osoby
	 * @return Integer
	 */
	public int getAge() { return age; }

	/**
	 * Zmena veku osoby s respektovanim pravidla <strong>0 < age < {@value #MAX_AGE}  </strong>
	 * @param age Integer
	 * @return [boolean] <strong>TRUE</strong> pokud je vek validni, jinak <strong>FALSE</strong>
	 */
	public boolean setAge(int age) { 
		if(age > 0 && age < MAX_AGE){
			this.age = age;
			return true;
		}else {
			return false;
		}
	}

	/**
	 * Partner
	 * 
	 * @return
	 */
	public Person getPartner() { return partner; }
	
	/**
	 * Nastavi partnera
	 * 
	 * @param partner
	 */
	public void setPartner(Person partner) { this.partner = partner; }

	/**
	 * Rodic - otec.
	 * 
	 * @return Person
	 */
	public Person getDad() { return dad; }
	/**
	 * Rodic - otec.
	 * 
	 * @return Person
	 */
	public Person getFather() { return dad; }
	
	public void setDad(Person p){ this.dad = p; }
	
	/**
	 * Rodic - matka.
	 * 
	 * @return Person
	 */
	public Person getMom() { return mom; }
	/**
	 * Rodic - matka.
	 * 
	 * @return Person
	 */
	public Person getMother() { return mom; }
	
	public void setMom(Person p){ this.mom = p; }
	

	/**
	 * Vrati seznam rodicu
	 * 
	 * @return
	 */
	public LinkedList<Person> getParent(){
		LinkedList<Person> parents = new LinkedList<Person>();
		parents.add(getDad());
		parents.add(getMom());
		return parents;
	}
	
	/**
	 * Vrati seznam potomku
	 * 
	 * @return Person
	 */
	public LinkedList<Person> getChildren() {
		return children;
	}
	
	/**
	 * Prida noveho potomka
	 * 
	 * @param child
	 * @return
	 */
	public boolean addChild(Person child){
		return children.add(child);
	}
	
	public boolean removeChild(Person child){
		return children.remove(child);
	}
	
	public boolean removeAllChild(){
		return children.removeAll(children);
	}

	@Override
	public String toString() {
		return "Person [id=" + id + ", name=" + name + ", lastname=" + lastname
				+ ", father=" + (dad == null ? "null" : dad.getId()) + ", mother=" + (mom == null ? "null" : mom.getId())
				+ ", partner=" + (partner == null ? "null" : partner.getId()) + ", children= "+children.size()+"]";
	}
	
	public String toPrint(){
		return "" +this.getFullName()+ "(" +id+ "): " +
				"father is " + (dad == null ? "null" : dad.getFullName() +"(" +dad.getId()+ ")") + 
				", mother is " + (mom == null ? "null" : mom.getFullName() +"(" +mom.getId()+ ")") +
				", partner is " + (partner == null ? "null" : partner.getFullName() +"(" +partner.getId()+ ")") +
				", have "+children.size()+" children]";
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + id;
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		Person other = (Person) obj;
		if (id != other.id)
			return false;
		return true;
	}
	
}
